// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"os"

	"github.com/spf13/cobra"
)

// keyCmd represents the key command.
var keyCmd = &cobra.Command{
	Use:   "key",
	Short: "Manage private keys used by this program.",
	Long:  "Manage private keys used by this program.",
}

// keyGenerateCmd represents the key generation sub-command.
var keyGenerateCmd = &cobra.Command{
	Use:   "generate",
	Short: "Generate a private key.",
	Long:  "Generate a private key named based on the selected user.",
	Run:   generateKey,
}

// generateKey parses arguments to generate a private key.
func generateKey(cmd *cobra.Command, args []string) {
	os.Exit(doGenerateKey())
}

// doGenerateKey generates a private key  named based on the selected user.
func doGenerateKey() int {
	client, err := GetClient()
	if err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	if err := client.CreateKey(false); err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	Logger.Noticef("Successfully generated a key.")
	return 0
}

func init() {
	RootCmd.AddCommand(keyCmd)

	keyCmd.AddCommand(keyGenerateCmd)
}
