// Copyright 2016 Orus.io. All rights reserved.

package cmd

import (
	"fmt"
	"os"

	"github.com/m4rw3r/uuid"
	"github.com/olekukonko/tablewriter"
	"github.com/spf13/cobra"

	"bitbucket.org/orus-io/go-xbus/api"
)

var pmCmd = &cobra.Command{
	Use:   "pm",
	Short: "Postmortem process management",
	Run: func(cmd *cobra.Command, args []string) {
		c, err := GetClient()
		if err != nil {
			Logger.Errorf("%s", err)
			os.Exit(1)
		}
		list, err := c.PMProcessQuery(api.LogWarning, false, []uuid.UUID{}, false)
		if err != nil {
			Logger.Errorf("%s", err)
			os.Exit(1)
		}
		table := tablewriter.NewWriter(os.Stdout)
		table.SetHeader([]string{"id", "level", "status", "comment"})
		for _, p := range list {
			table.Append([]string{
				p.ID.String(),
				p.Level.String(),
				p.Status.String(),
				p.Comment,
			})
		}
		table.Render()
	},
}

var pmResetCmd = &cobra.Command{
	Use:   "reset",
	Short: "Reset a postportem process status to 'new'",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(setStatus(
			uuid.MustFromString(args[0]),
			api.PostMortemNew,
			args[1]))
	},
}

var pmOpenCmd = &cobra.Command{
	Use:   "open",
	Short: "Indicate a postportem process is being addressed",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(setStatus(
			uuid.MustFromString(args[0]),
			api.PostMortemOpened,
			args[1]))
	},
}

var pmCloseCmd = &cobra.Command{
	Use:   "close",
	Short: "Indicate a postportem process is closed",
	Run: func(cmd *cobra.Command, args []string) {
		fmt.Println(args)
		os.Exit(setStatus(
			uuid.MustFromString(args[0]),
			api.PostMortemClosed,
			args[1]))
	},
}

func setStatus(processID uuid.UUID, status api.PostMortemStatus, comment string) int {
	c, err := GetClient()
	if err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	if err := c.PMProcessSetStatus(processID, status, comment); err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	return 0
}

func init() {
	RootCmd.AddCommand(pmCmd)
	pmCmd.AddCommand(pmResetCmd)
	pmCmd.AddCommand(pmOpenCmd)
	pmCmd.AddCommand(pmCloseCmd)
}
