// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"encoding/json"
	"time"

	"bitbucket.org/orus-io/go-xbus/api"
)

func (s *Service) handlePMProcessQuery(subject, reply string, req api.ControlPMProcessQueryReq) {
	var rep api.ControlPMProcessQueryRep
	rep.Response.Status = api.StatusOK

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	statuses := []api.PostMortemStatus{
		api.PostMortemNew,
		api.PostMortemOpened,
	}
	if req.IncludeClosed {
		statuses = append(statuses, api.PostMortemClosed)
	}
	list, err := s.o.ProcessManager.ListPostMortem(req.Level, statuses...)
	if err != nil {
		panic(err)
	}
	rep.PMProcessList = make([]api.PMProcess, 0, len(list))
	for _, process := range list {
		pmstate, err := process.GetPostMortemState()
		if err != nil {
			panic(err)
		}
		state := s.o.ProcessStorage.GetState(process.ID)
		b, err := json.Marshal(state)
		if err != nil {
			panic(err)
		}
		rep.PMProcessList = append(rep.PMProcessList,
			api.PMProcess{
				Process: api.Process{
					ID:                process.ID,
					TriggerEmitterID:  process.TriggerEmitterID,
					TriggerEnvelopeID: process.TriggerEnvelopeID,
					GraphID:           process.Graph.ID,
					Status:            process.GetStatus(),
					State:             string(b),
				},
				Level:   pmstate.Level,
				Status:  pmstate.Status,
				Comment: pmstate.Comment,
			})
	}
}

func (s *Service) handlePMProcessSetStatus(subject, reply string, req api.ControlPMProcessSetStatusReq) {
	var rep = api.Response{Status: api.StatusOK}

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	process, err := s.o.ProcessManager.Get(req.ProcessID)
	if err != nil {
		panic(err)
	}

	pmState, err := process.GetPostMortemState()
	if err != nil {
		rep = api.ClientError("%s", err)
		return
	}

	pmState.Status = req.Status
	pmState.Comment = req.Comment
	pmState.Changed = time.Now()

	if err := process.SetPostMortemState(pmState); err != nil {
		rep = api.ClientError("%s", err)
	}
}
