// Copyright 2016 Orus.io. All rights reserved.

package control

import (
	"encoding/json"
	"fmt"

	"bitbucket.org/orus-io/go-xbus/api"
	"gopkg.in/yaml.v2"
)

func (s *Service) handleProcessQuery(subject, reply string, req api.ControlProcessQueryReq) {
	var rep api.ControlProcessQueryRep

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	pList := s.o.ProcessStorage.Query(req.Filter)

	for _, p := range pList {
		state := s.o.ProcessStorage.GetState(p.ID)
		b, err := json.Marshal(state)
		if err != nil {
			panic(err)
		}
		rep.ProcessList = append(rep.ProcessList, api.Process{
			ID:                p.ID,
			TriggerEmitterID:  p.TriggerEmitterID,
			TriggerEnvelopeID: p.TriggerEnvelopeID,
			GraphID:           p.GraphID,
			Status:            state.Status,
			State:             string(b),
		})
	}

	rep.Status = api.StatusOK
}

func (s *Service) handleProcessControl(subject, reply string, req api.ControlProcessControlReq) {
	var rep api.ControlProcessControlRep

	s.TaskAdd()
	defer s.SendReply(reply, &rep)
	var err error
	switch req.Command {
	case "pause":
		err = s.o.ProcessStorage.SetStatus(req.ProcessID, api.ProcessPaused)
	case "resume":
		err = s.o.ProcessStorage.SetStatus(req.ProcessID, api.ProcessRunning)
	default:
		err = fmt.Errorf("Invalid command: %s", err)
	}
	if err != nil {
		rep.Response = api.ClientError("Error: %s", err)
		return
	}
	rep.Status = api.StatusOK
}

func (s *Service) handleProcessExport(subject, reply string, req api.ControlProcessExportReq) {
	var rep api.ControlProcessExportRep

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	for _, pid := range req.ProcessIDs {
		p, err := s.o.ProcessManager.Get(pid)
		if err != nil {
			panic(err)
		}

		export, err := p.Export(s.o.EnvelopeStorage, s.Log)
		if err != nil {
			panic(err)
		}

		b, err := yaml.Marshal(export)
		if err != nil {
			panic(err)
		}

		rep.ProcessList = append(rep.ProcessList, string(b))
	}

	rep.Response.Status = api.StatusOK
}

func (s *Service) handleProcessPurge(subject, reply string, req api.ControlProcessPurgeReq) {
	var rep api.ControlProcessPurgeRep

	s.TaskAdd()
	defer s.SendReply(reply, &rep)

	for _, pid := range req.ProcessIDs {
		err := s.o.ProcessManager.Purge(pid)
		if err != nil {
			panic(err)
		}
	}

	rep.Status = api.StatusOK
}
