package storage

import (
	"fmt"
	"strings"

	"gopkg.in/yaml.v2"

	"github.com/m4rw3r/uuid"
)

func (s ReceptionStatus) String() string {
	switch s {
	case ReceptionPending:
		return "pending"
	case ReceptionOngoing:
		return "ongoing"
	case ReceptionDone:
		return "done"
	case ReceptionClosed:
		return "closed"
	default:
		return ""
	}
}

// String serialize a ProcessNode
func (n ProcessNode) String() string {
	return n.ActorID.String() + "/" + n.NodeID
}

// ProcessNodeFromString deserialize a ProcessNode
func ProcessNodeFromString(s string) (ProcessNode, error) {
	var (
		n   ProcessNode
		err error
	)
	splitted := strings.Split(s, "/")
	if len(splitted) != 2 {
		return n, fmt.Errorf("Invalid processnode: %s. Expects '<actoruuid>/<nodeid>'", s)
	}
	n.ActorID, err = uuid.FromString(splitted[0])
	if err != nil {
		return n, err
	}
	n.NodeID = splitted[1]
	return n, nil
}

// MarshalText implements encoding.TextMarshaler for ProcessNode
func (n ProcessNode) MarshalText() ([]byte, error) {
	return []byte(n.String()), nil
}

// UnmarshalText implements encoding.TextUnmarshaler for ProcessNode
func (n *ProcessNode) UnmarshalText(text []byte) (err error) {
	*n, err = ProcessNodeFromString(string(text))
	return
}

// Fmt returns a formatted string of the state
func (s ProcessState) Fmt() string {
	b, err := yaml.Marshal(s)
	if err != nil {
		panic(err)
	}
	return string(b)
}
