package sql

import (
	"github.com/orus-io/yago"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/xbus/storage/sql/model"
)

func must(err error) {
	if err != nil {
		panic(err)
	}
}

// TxHelper wraps a tx and make it easier to enforce commit or rollback
type TxHelper struct {
	*yago.Tx
	complete bool
	logger   xbus.Logger
}

// MustBegin starts a transaction and wrap it
func MustBegin(db *yago.DB, logger xbus.Logger) *TxHelper {
	tx, err := db.Begin()
	if err != nil {
		logger.Errorf("Cannot start a transaction: %s", err)
		panic(err)
	}
	return &TxHelper{tx, false, logger}
}

// Commit commits the transaction
func (tx *TxHelper) Commit() error {
	err := tx.Tx.Commit()
	tx.complete = true
	if err != nil {
		tx.logger.Errorf("Error on commit: %s", err)
	}
	return err
}

// MustCommit commits of rollback
func (tx *TxHelper) MustCommit() {
	err := tx.Commit()
	if err != nil {
		panic(err)
	}
}

// Rollback rollbacks and log error if any
func (tx *TxHelper) Rollback() error {
	err := tx.Tx.Rollback()
	tx.complete = true
	if err != nil {
		tx.logger.Errorf("Error on rollback: %s", err)
	}
	return err
}

// RollbackIfOpened rollback the transaction is still open
func (tx *TxHelper) RollbackIfOpened() {
	if !tx.complete {
		tx.Rollback()
	}
}

// Base provides a helpful base for sql-based storage implementation
type Base struct {
	model  *model.Model
	db     *yago.DB
	logger xbus.Logger
}

// MustBegin stats a new transaction
func (b Base) MustBegin() *TxHelper {
	return MustBegin(b.db, b.logger)
}
