// Copyright 2016 Orus.io. All rights reserved.

package api

import (
	"time"

	"github.com/m4rw3r/uuid"
)

// LogLevel is the level of a log message
//go:generate enum-marshaler -type LogLevel -prefix Log -lower -tests
type LogLevel int

const (
	// LogNotice is for notice messages
	LogNotice LogLevel = iota
	// LogWarning is for warning messages. They do not imply a process error
	// but will be appear as warning in the process state
	LogWarning
	// LogError if for error messages. They imply a 'error' state of the Process
	LogError
)

// LogMessage is a logging message of a process
type LogMessage struct {
	Time  time.Time
	Level LogLevel `json:"level"`
	Text  string   `json:"text"`
}

// LogEntry is a LogMessage with a context in which it happened
type LogEntry struct {
	EnvelopeID uuid.UUID
	ActorID    uuid.UUID
	ProcessID  uuid.UUID
	NodeID     string
	Message    LogMessage
}
