package demo

import (
	"sync"

	"bitbucket.org/orus-io/go-xbus"
)

// NewPrintToConsoleConsumer is a PrintToConsoleConsumer factory
func NewPrintToConsoleConsumer(actor *xbus.Actor) xbus.ActorService {
	return &PrintToConsoleConsumer{actor, nil}
}

// PrintToConsoleConsumer is a consumer that prints the received items on
// the console
type PrintToConsoleConsumer struct {
	actor *xbus.Actor
	sub   *xbus.Subscription
}

// Startup initializes the subscription
func (c *PrintToConsoleConsumer) Startup() error {
	sub, err := c.actor.Subscribe(func(er *xbus.EnvelopeReceiver) {
		var wg sync.WaitGroup
		for _, eventType := range er.EventTypes() {
			wg.Add(1)
			ch := er.GetItemChan(eventType)
			go func(eventType string) {
				for item := range ch {
					c.actor.GetClient().Log.Noticef(
						"print-to-console - %s: %s", eventType, string(item))
				}
				wg.Done()
			}(eventType)
		}
		wg.Wait()
		c.actor.ProcessingSuccess(er.Context(), er.ID)
	})
	if err != nil {
		return err
	}
	c.sub = sub
	return nil
}

// Shutdown unsubscribes the consumer
func (c *PrintToConsoleConsumer) Shutdown() error {
	err := c.sub.Unsubscribe()
	c.sub = nil
	return err
}

func init() {
	xbus.RegisterActorService("demo.print-to-console", NewPrintToConsoleConsumer)
}
